
package uk.ac.ebi.msd.vamsas.vamav.documentmanagement;

import uk.ac.ebi.msd.vamsas.vamav.VAMAVClient;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.exceptions.IncorrectDocumentException;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.exceptions.UnableToProcessException;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping_domain;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping_structure;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.exceptions.UnableToGroupSequencesException;
import uk.ac.ebi.msd.vamsas.vamav.exceptions.IncorrectVamavModuleInitialisation;
import uk.ac.ebi.msd.vamsas.vamav.util.logger.VAMAVLogger;
import uk.ac.ebi.msd.vamsas.vamav.util.logger.VAMAVLoggerImpl;
import uk.ac.ebi.msd.vamsas.vamav.util.mapping.MappingManager;
import uk.ac.ebi.msd.vamsas.vamav.util.properties.PropertyManager;
import org.vamsas.objects.core.Alignment;
import org.vamsas.objects.core.AlignmentAnnotations;
import org.vamsas.objects.core.AlignmentSequence;
import org.vamsas.objects.core.AnnotationElement;
import org.vamsas.objects.core.ApplicationData;
import org.vamsas.objects.core.DataSet;
import org.vamsas.objects.core.DbRef;
import org.vamsas.objects.core.Entry;
import org.vamsas.objects.core.Provenance;
import org.vamsas.objects.core.Sequence;
import org.vamsas.objects.core.SequenceType;
import org.vamsas.objects.core.Tree;
import org.vamsas.objects.core.User;
import org.vamsas.objects.core.VAMSAS;
import org.vamsas.objects.core.VamsasDocument;

import org.vamsas.client.IClient;
import org.vamsas.client.IClientDocument;
import org.vamsas.client.simpleclient.IdFactory;
import org.vamsas.client.simpleclient.SimpleDocument;
import org.vamsas.client.simpleclient.VamsasArchive;
import org.vamsas.client.simpleclient.ClientDocument;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.TimeZone;

import MSD.vamav.common.AVActiveSite;
import MSD.vamav.common.AVSequence;
import MSD.vamav.common.Highlight;
import MSD.vamav.common.ResidueAnnotation;

/**
 * <p>Processor for Vamsas Document, to extract data for calling an AstexViewer@MSD-EBI, for displaying the provided data</p>
 * 
 * <p>Grouping mode :</p>
 * <ul>
 *  <li>1. per structure</li>
 *  <li>2. per PFAM domain</li>
 *  </ul>
 * 
 */
public class VamsasDocumentProcessor implements uk.ac.ebi.msd.vamsas.vamav.VamaVModule {

	
	//private IClientDocument doc = null;
	private IClient client = null;
	private  VAMAVLogger logger = new VAMAVLoggerImpl(this.getClass());
	
	private final String modulePrefix = "documentprocessor";
	private SequenceGrouping  groupingManager = null;
	private int moleculeNumber = 1;
	private int highlightNumber = 0;
	private boolean hasAlignmentSequences = false; 
	private MappingManager mappingManager = null;
	private String activeSiteDescription = "Active Site";
	
	/**
	 * Retrieves the  modulePrefix attribut
	 *
	 * @return Returns the modulePrefix.
	 */
	public String getModulePrefix()
		{
			return this.modulePrefix;
		}
	
	public void init(Properties moduleProperties) throws IncorrectVamavModuleInitialisation
		{
			PropertyManager propManager = new PropertyManager();
			Properties props = propManager.getModuleProperties(moduleProperties, this.groupingManager.getModulePrefix());
			this.groupingManager.init(props);
			propManager = null;
		}
	
	/**
	 * 
	 * Constructor for a given document 
	 * @param document Vamsas Document to process.
	 * @param _mappingManager mapping manager
	 * @throws IncorrectDocumentException if the given document is incorrect
	 */
	public VamsasDocumentProcessor (IClient _client, MappingManager _mappingManager ) throws IncorrectDocumentException
		{
			if(_client == null   ) throw new IncorrectDocumentException();
			
			this.client = _client;
			this.logger.setSession(this.client.getSessionUrn());
			this.mappingManager = _mappingManager;
			this.mappingManager.setSession(((VAMAVClient)this.client).getVAMAVSession());
			this.groupingManager = new SequenceGrouping_structure(this.mappingManager, ((VAMAVClient)this.client).getVAMAVSession());
			//this.logger = new  VAMAVLoggerImpl(this.getClass(),this.client.getSessionUrn() );
			
			//this.groupingManager.load();
		}
	
	/**
	 * 
	 * Constructor for a given document 
	 * @param document Vamsas Document to process.
	 * @param _mappingManager mapping manager
	 * @param groupingMode grouping Mode number, for sequence grouping (structure, domain)
	 * @throws IncorrectDocumentException if the given document is incorrect
	 */
	public VamsasDocumentProcessor (IClient _client, MappingManager _mappingManager, int groupingMode) throws IncorrectDocumentException
		{
			if(_client == null ) throw new IncorrectDocumentException();
			
			this.client = _client;
			this.logger.setSession(this.client.getSessionUrn());
			
			
			((VAMAVClient)this.client).getVAMAVSession().setGroupingMode(groupingMode);
			this.mappingManager = _mappingManager;
			this.mappingManager.setSession(((VAMAVClient)this.client).getVAMAVSession());
			this.mappingManager.setSession(((VAMAVClient)this.client).getVAMAVSession());
			if( groupingMode == 0)
				{
					this.groupingManager = new SequenceGrouping_structure(this.mappingManager, ((VAMAVClient)this.client).getVAMAVSession());
				}
			else
				if( groupingMode == 1)
				{
					this.groupingManager = new SequenceGrouping_domain(this.mappingManager, ((VAMAVClient)this.client).getVAMAVSession());
				}
				else
					// default
					this.groupingManager = new SequenceGrouping_structure(this.mappingManager, ((VAMAVClient)this.client).getVAMAVSession());
			
		
			//this.logger = new  VAMAVLoggerImpl(this.getClass(),this.client.getSessionUrn() );
			
			//this.groupingManager.load();
		}
/**
 * <p>Process a document to extract element for AstexViewer@MSD-EBI</p>
 * 
 */
    public Hashtable<String, Object> process( )  throws IncorrectDocumentException, UnableToProcessException
    	{        
    		Hashtable<String, Object>  parameters = new Hashtable<String, Object>();
    		//int moleculeNumber = 1;
			IClientDocument  doc =  null;
    		try
    			{
    		   		doc = this.client.getClientDocument();
    			}
    		catch (IOException ioe)
    			{
    				this.logger.sessionError("Unable to access to the VAMSAS document. Can not process.",ioe);
					throw new IncorrectDocumentException(ioe);
    			}
    		if( doc == null)
    			{
    				this.logger.sessionError("The document is null. Can not process it.");
    				throw new IncorrectDocumentException();
    			}
    		this.logger.sessionInfo("The processing vamsas document.");
    		try	
    			{
    				VAMSAS[] roots = doc.getVamsasRoots();
    				if( roots != null && roots.length>0)
    					for (int rootNumber = roots.length-1; rootNumber>=0; rootNumber--)
    						{
    							VAMSAS root = roots[rootNumber];
    							if (root == null) continue;
    							Tree [] trees = root.getTree();
    							//treating Trees
    							if (trees !=null)
    								{
    									for (int treeNumber=0; treeNumber< trees.length; treeNumber++)
    										{
    											Tree aTree = trees[treeNumber];
    											if( aTree == null) continue;
    											parameters = this.addParameters(this.processTree(aTree), parameters);
    										}
    								}
    							else
    								{
    									this.logger.sessionDebug("No Tree found.");
    								}
    							int setCount = root.getDataSetCount();
    							this.logger.sessionDebug("DataSet "+setCount);
    							if (setCount>0)
    								{ 
    									this.hasAlignmentSequences = false;
    									for (int setNumber=0; setNumber< setCount; setNumber++)
    										{
    											//                    		treating DataSet
    											this.logger.sessionDebug("DataSet " +setNumber );
    											DataSet set = root.getDataSet(setNumber);
    											if (set == null)
    												{
    													this.logger.sessionDebug("data set is null?? "+setNumber);
    													continue;
    												}
    											parameters = this.addParameters(this.processDataSet(set), parameters);
    										}
    								}  
    						}
    			}
    		catch (Exception e)
    			{
    				this.logger.sessionError("VamsasDocumentProcessor process- Error during Document processing.",e);
    				parameters = null;
    			}
    		if (parameters!=null) this.logger.sessionDebug("parameter count "+parameters.size());
    		//store PDB 
    		//this.logger.sessionInfo("Save computed data");
    		try
    			{
    				File atrFile = this.groupingManager.storeGroups();
     //  this.groupingManager.getSequenceFiles()
 //     add atribute file parameter
    				if(atrFile != null)
    					{
							this.logger.sessionDebug("Atribute file "+atrFile.getName());
    						parameters.put("attribute", atrFile);
    					}
    				else 
    					this.logger.sessionError("No atribute file created.");
       
    		   //addsequenceFile
    		   ArrayList<File>seqFiles = this.groupingManager.getSequenceFiles();
    		   if(seqFiles != null && seqFiles.size()>0)
    			   {
    				   for (int fileNumber = seqFiles.size() - 1;fileNumber >=0; fileNumber -- )
    					   {
    						   File file = seqFiles.get(fileNumber);
    						   if( file == null) continue;
    						   this.logger.sessionDebug("VAMSASSequence"+(fileNumber+1)+": add sequence file to parameter "+file.getName());
    						   parameters.put("VAMSASSequence"+(fileNumber+1), file);
    					   }
    	   			}
    	   }
       catch (UnableToGroupSequencesException ue)
    	   {
    		   this.logger.sessionError("VamsasDocumentProcessor StoreGroups - Unable to group sequences.", ue);
    		   throw new UnableToProcessException(ue);
    	   }
    	   
      this.groupingManager = null;
      this.logger.sessionDebug("process "+(parameters==null?0:parameters.size()));
       return parameters;
    } 

    /**
     * Process Vamsas Tree object
     * Currently do nothing
     * @param atree the tree to process
     * @return a list of parameters to call an AstexViewer@MSD-EBI client
     */
    private Hashtable <String, Object> processTree(Tree atree)
    	{
    		return null;
    	}
    
    
    private Hashtable <String, Object> processDataSet(DataSet aDataSet) throws UnableToGroupSequencesException
    	{
    		if (aDataSet == null)
    			{
					this.logger.sessionDebug("data set is null??");
					return null;
    			}
    		Hashtable <String, Object> parameters = new Hashtable <String, Object>();
    		int alignCount = aDataSet.getAlignmentCount();
    		if (alignCount>0)
    			{
    				for (int algNumber=0; algNumber< alignCount; algNumber++)
    					{
    						this.logger.sessionDebug("Alignment " +algNumber );
							Alignment align = aDataSet.getAlignment(algNumber);
							if (align == null) continue; //should not happen
							Hashtable <String, Object> newparameters =this.ProcessAlignment(align);
							this.logger.sessionDebug("end ab aligment  "+(newparameters==null?0:newparameters.size()));
							parameters = this.addParameters(newparameters, parameters);
							this.logger.sessionDebug("end ab aligment  "+(parameters==null?0:parameters.size()));
    					}
    				this.logger.sessionDebug("end aligments  "+(parameters==null?0:parameters.size()));
    			}	
    		int sequenceCount = aDataSet.getSequenceCount();
    		if (!this.hasAlignmentSequences && sequenceCount > 0 )
    			{  
    				this.logger.sessionInfo("No alignment has been found. Process DataSet sequences."+sequenceCount);
    				for (int seqNumber=0; seqNumber< sequenceCount; seqNumber++)
    					{
    						Sequence seq = aDataSet.getSequence(seqNumber);
    						if (seq == null)
    							{
    								this.logger.sessionDebug("sequence  is null??");
    								continue;
    							}
    						
    						this.logger.sessionDebug(seq.getId());
    						DbRef[] dbRefs = seq.getDbRef();
    						boolean idFound = false;
    						if( dbRefs != null && dbRefs.length >0)
    							{
    								for (int dbRefNumber = dbRefs.length - 1; dbRefNumber >=0; dbRefNumber-- )
    									{
    										DbRef ref = dbRefs[dbRefNumber];
    										if( ref == null) continue;
    										
    										String acID = ref.getAccessionId();
											if( acID == null)
												{
													continue;
												}
											String source = ref.getSource() ;
											if( source== null || source.length()<1 )
												{
													continue;
												}
											
											Hashtable <String, Object> tempParameters = this.groupingManager.addSequence(seq.getSequence(), seq.getName(), acID, source);
											if (tempParameters != null && tempParameters.size()>0)
												{
													this.logger.sessionDebug("parameter count "+parameters.size());
													String pdbids = (String) tempParameters.remove(this.groupingManager.getPdbidsproperty());
															
													if(pdbids!=null && pdbids.length()>0)
														{
															//TODO check if the ref does not have already the new DB reference
															this.logger.sessionDebug("Adding pdb IDs to doc "+pdbids );
															DbRef dbref = new DbRef();
															dbref.setSource("PDB");
															dbref.setAccessionId(pdbids);
															seq.addDbRef(dbref);
															//seq.setRefid(ref)
														}
													parameters = addParameters(tempParameters, parameters);
													this.logger.sessionDebug("end add sequence "+(parameters==null?0:parameters.size()));
													idFound = true;
													this.hasAlignmentSequences = true;
												}
												
											/*String pdbids = (String) tempParameters.remove(this.groupingManager.getPdbidsproperty());
												String id = ref.getAccessionId();
    										if(id == null || id.length()<1 ) continue;
    										//suppose it is  a PDB ID (should be like PDBID:CHAIN)
    										//We don t handle chain name , remove id
    										int pos = id.indexOf(":");
    										if (pos>=0)
    											{//there is a match
    												id = id.substring(0,pos);
    											}
    											PDBEntry entry = SequenceToPDB_MSDLite.getPDBFileFromPDBID(id);
    											if( entry != null)
    											{
    											idFound = true;
    											if( !parameters.containsValue(entry))
    											/*parameters.add(entry);*/
											/*	{
													parameters.put("molecule"+moleculeNumber,entry);
													parameters.put("name"+moleculeNumber, entry.getPdbID());
													moleculeNumber++;
												}
								Grouping.addSequenceToGroup(entry.getPdbID(), seq);*/
    									
    									}	
    							}
    						if (!idFound)
								{
									Hashtable<String, Object>tempParameters = this.processSequence(seq);
									if (tempParameters != null)
									{
									//parameters = this.addParameters(this.processSequence(seq), parameters);
									String pdbids = (String) tempParameters.remove(this.groupingManager.getPdbidsproperty());
									if(pdbids!=null && pdbids.length()>0)
										{
											//TODO check if the ref does not have already the new DB reference
											this.logger.sessionDebug("Adding pdb IDs to doc "+pdbids );
											DbRef ref = new DbRef();
											ref.setSource("PDB");
											ref.setAccessionId(pdbids);
											seq.addDbRef(ref);
											//seq.setRefid(ref)
										}
										parameters = this.addParameters(tempParameters, parameters);
									}
								}
    					}
    			}
    		this.logger.sessionDebug("ProcessDataSet "+(parameters==null?0:parameters.size()));
    		return parameters;
    	}
   	
   
    /**
     * Process an alignment to extract sequences for grouping
     * 
     * @param align alignment Object to process
     */
    private Hashtable <String, Object> ProcessAlignment(Alignment align) throws UnableToGroupSequencesException
    	{
    		if (align == null) return null; //should not happen
			int seqCount = align.getAlignmentSequenceCount();
			Hashtable <String, Object> parameters = null;
			if( seqCount>0 )
				{
					parameters = new Hashtable <String, Object>();
					String sequenceName =  null;
					for (int seqNumber=0; seqNumber< seqCount; seqNumber++)
						{
							this.logger.sessionDebug("alig seq " +seqNumber );
							AlignmentSequence seq = align.getAlignmentSequence(seqNumber);
							if (seq == null) continue; //should not happen
							Sequence refSeq = (Sequence) seq.getRefid();
							boolean notFound = true;
							if (refSeq != null)
								{
									if (refSeq.getDbRefCount()>0)
										{
											int dbRefNumber = 0;
											int refCount = refSeq.getDbRefCount();
											while (notFound && dbRefNumber<refCount)
												{
													DbRef dbRef= refSeq.getDbRef(dbRefNumber++);
													this.logger.sessionDebug("dbref  " +(dbRefNumber -1) );
													if(dbRef == null) continue;
													String acID = dbRef.getAccessionId();
													if( acID == null)
														{
															continue;
														}
													String source = dbRef.getSource() ;
													if( source== null || source.length()<1 )
														{
															continue;
														}
													
													Hashtable <String, Object> tempParameters = this.groupingManager.addSequence(seq.getSequence(), seq.getName(), acID, source);
													
													if (tempParameters != null && tempParameters.size()>0)
														{
															this.logger.sessionDebug("parameter count "+parameters.size());
															String pdbids = (String) tempParameters.remove(this.groupingManager.getPdbidsproperty());
															if(pdbids!=null && pdbids.length()>0)
																{
																	//TODO check if the ref does not have already the new DB reference
																	this.logger.sessionDebug("Adding pdb IDs to doc "+pdbids );
																	DbRef ref = new DbRef();
																	ref.setSource("PDB");
																	ref.setAccessionId(pdbids);
																	refSeq.addDbRef(ref);
																	//seq.setRefid(ref)
																}
															parameters = addParameters(tempParameters, parameters);
															notFound = false;
															hasAlignmentSequences = true;
															this.logger.sessionDebug("parameter count "+parameters.size());
														}
													//if (sequenceName ==null) sequenceName = entry.getPdbID()+"_";
												}
										}
									else
										{ /*System.out.println("No DbRef");*/
										}
								}
							if (notFound)
								{
									Hashtable<String, Object> tempParameters = processSequence(seq);
									if (tempParameters != null)
											{
										/*	String pdbids = (String) tempParameters.remove(this.groupingManager.getPdbidsproperty());
											if(pdbids!=null && pdbids.length()>0)
													{
													//TODO check if the ref does not have already the new DB reference
													this.logger.sessionDebug("Adding pdb IDs to doc "+pdbids );
													DbRef ref = new DbRef();
													ref.setSource("PDB");
													ref.setAccessionId(pdbids);
													seq.addDbRef(ref);
												//seq.setRefid(ref)
												}*/
											parameters = this.addParameters(tempParameters, parameters);
											}
								}
						}       	
						//treat AnnotationElement for highlight
					int annotationCount = align.getAlignmentAnnotationsCount();
					this.logger.sessionDebug("annotationCount "+ annotationCount);
					for (int annotationNumber = annotationCount-1; annotationNumber>=0; annotationNumber--)
						{
							this.logger.sessionDebug("annotationNumber "+annotationNumber);
							AlignmentAnnotations annotation = align.getAlignmentAnnotations(annotationNumber);
							if (annotation == null) continue;
							int elementCount =  annotation.getAnnotationElementCount();
							for (int elementNumber = elementCount-1; elementNumber>=0; elementNumber--)
								{
									this.logger.sessionDebug("annotationElement  "+elementNumber);
									AnnotationElement elt = annotation.getAnnotationElement(elementNumber);
									if (elt == null) continue;
									parameters = this.addParameters(this.processAnnotationElement(elt), parameters);
								}
						}
				}
			this.logger.sessionDebug("ProcessAlignment "+(parameters==null?0:parameters.size()));
			return parameters;
    	}
    
   private Hashtable <String, Object> processAnnotationElement( AnnotationElement annotation)
    	{
    		if (annotation == null) return null;
			int secStructure = -1;//unknown
			Hashtable <String, Object> parameters = null;
			try
				{
					secStructure = new Integer(annotation.getSecondaryStructure()).intValue();
				}
			catch (NumberFormatException nfExcep)
				{
					this.logger.sessionWarn("Unknown secondary sequence: "+ annotation.getSecondaryStructure());
					secStructure = -1;
				}
			if (annotation.getDescription().contains("breakpoint") || annotation.getDescription().contains("Breakpoint"))
					{
						Highlight hgl = new Highlight (annotation.getDisplayCharacter().charAt(0), annotation.getDescription(), secStructure, null, annotation.getPosition());
						//Hashtable<String, Object> temp
						parameters = new Hashtable<String, Object>();
						//temp
						parameters.put("highlightT0",hgl);
						this.groupingManager.addBreakpoint(annotation);
					}
			//this.groupingManager.addBreakpoint(annotation);
			return parameters;
    	}
    
    private Hashtable <String, Object> processSequence(SequenceType sequence) throws UnableToGroupSequencesException
    {
    	if (sequence == null) return null;
		Hashtable <String, Object> tempParameters =  this.groupingManager.addSequence(sequence.getSequence(), sequence.getName());
		
    	//return null;
		
		return tempParameters;
    }
/**
 * <p>Does ...</p>
 * 
 */
    public void updateDocument(Hashtable<String, Object> parameters) 
    	{        
    		this.logger.sessionDebug("Updating Document");
    		if( parameters == null || parameters.size()<1)
    			{
    				this.logger.sessionWarn("UpdateDocument: no object provided: No update.");
    				return ;
    			}
			VAMSAS[] roots = null;
    		try
    			{
    				roots = this.client.getClientDocument().getVamsasRoots();
    			}
    		catch (IOException ioe)
    			{
					this.logger.sessionWarn("Add active site: The document does not exist. Can not add Active site.");
					return;
    					
    			}
    		if (roots == null || roots.length<1)
    			{
    				this.logger.sessionWarn("Add active site: The document is empty. Can not add Active site.");
    				return;
    			}
    		for (int rootNumber = roots.length-1; rootNumber >=0 &&  parameters != null&& parameters.size()>0; rootNumber-- )
    			{
    				VAMSAS root = roots[rootNumber];
    				if (root == null)
    					{
    						continue;
    					}
    				updateVAMSAS(root,parameters);
    			}
    	} 
    
    /**
     * Updates a VAMSAS entity with the given parameters
     * @param parameters  list of objects to update into the VAMSAS  tree
     */
    private void updateVAMSAS(VAMSAS root, Hashtable<String, Object> parameters) 
    	{        
    		this.logger.sessionDebug("Updating Vamsas");
    		if( parameters == null || parameters.size()<1)
    			{
    				this.logger.sessionWarn("UpdateVAMSAS: no object provided: No update.");
    				return ;
    			}
    		if(root == null )
    			{
    				this.logger.sessionWarn("UpdateVAMSAS: no VAMSAS XML Entity.");
    				return ;
    			}
    		for (int dataSetNumber = root.getDataSetCount() -1; dataSetNumber>=0&&  parameters != null&& parameters.size()>0; dataSetNumber--)
    			{
    				DataSet set  = root.getDataSet(dataSetNumber);
    				if( set == null) continue;
    				updateDataSet(set, parameters);
    					
    			}
    	}
    
    /**
     * Updates a DataSet entity with the given parameters
     * @param parameters  list of objects to update into the DataSet  tree
     */
    private void updateDataSet(DataSet root, Hashtable<String, Object> parameters) 
    	{        
    		this.logger.sessionDebug("Updating DataSet");
    		if( parameters == null || parameters.size()<1)
    			{
    				this.logger.sessionWarn("updateDataSet: no object provided: No update.");
    				return ;
    			}
    		if(root == null )
    			{
    				this.logger.sessionWarn("updateDataSet: no DataSet XML Entity.");
    				return ;
    			}
    		for (int alignmentNumber= root.getAlignmentCount()-1; alignmentNumber>=0&&  parameters != null&& parameters.size()>0; alignmentNumber-- )
    			{
    				Alignment align  = root.getAlignment(alignmentNumber);
    				if( align != null)
    					{
    						updateAlignment(align, parameters);
    					}
    			}
    	}

    /**
     * Updates a Alignment entity with the given parameters
     * @param parameters  list of objects to update into the Alignment  tree
     */
    private void updateAlignment(Alignment root, Hashtable<String, Object> parameters) 
    	{        
    		this.logger.sessionDebug("Updating ALignment");
    		if( parameters == null || parameters.size()<1)
    			{
    				this.logger.sessionWarn("updateAlignment: no object provided: No update.");
    				return ;
    			}
    		if(root == null )
    			{
    				this.logger.sessionWarn("updateAlignment: no DataSet XML Entity.");
    				return ;
    			}
    		this.logger.sessionDebug("list of parameters: "+parameters);
    		Hashtable<String, ArrayList<AVActiveSite>> sites = (Hashtable<String, ArrayList<AVActiveSite>>) parameters.get("MSD.vamav.common.AVActiveSite");
    		Hashtable<String, ArrayList<ResidueAnnotation>> residueAnnotations = (Hashtable<String, ArrayList<ResidueAnnotation>>) parameters.get("MSD.vamav.common.ResidueAnnotation");
        	try
        		{
    		if ((sites != null && sites.size()>0) || (residueAnnotations!= null && residueAnnotations.size()>0))
    			{
    				boolean hasSitesToUpdate = false;
    				if (sites!= null && sites.size()>0)
    					{
    						hasSitesToUpdate = true;
    					}
    				boolean hasAnnotationToUpdate = false;
    				if (residueAnnotations!= null && residueAnnotations.size()>0)
    					{
    						hasAnnotationToUpdate = true;
    					}
    				for (int seqNumber = root.getAlignmentSequenceCount() -1; seqNumber>=0; seqNumber --)
    					{
    						AlignmentSequence seq = root.getAlignmentSequence(seqNumber);
    						if( seq == null) continue;
    						String name = seq.getName();
    						if( name == null ) continue;
    						this.logger.sessionDebug("sequence name "+name);
    						name = name.trim().toLowerCase().intern();
    						if (hasSitesToUpdate)
    							{
    								if( sites.containsKey(name) )
    									{//found the alignment associated to an Active Site
    								//add the active site as annotation element
    							///	AlignmentAnnotations[] annotations = root.getAlignmentAnnotations();
    								//if (annotations == null)
    								//	{
    										this.addActiveSite(root, sites.get(name));
    										sites.remove(name);
    								//	}
    									}
    								else
    									{
    								//look for PDB ID
    								//usely active sites are associated with PDB chain.
    								//check any seq has a PDB ID with an active site.
    										Sequence seqRef = (Sequence)seq.getRefid();
    										if( !this.logger.isProduction())this.logger.sessionDebug(" sites "+sites);
    									//	this.logger.sessionDebug(" sites "+sites.keys());
    										if (seqRef != null)
    											{
	    										boolean found = false;
	    										int refNumber = seqRef.getDbRefCount() ;
	    										while (!found && refNumber>0)
	    											{
	    												refNumber --;
	    												DbRef ref = seqRef.getDbRef(refNumber);
	    												if( ref == null ) continue;
	    												String source = ref.getSource();
	    												this.logger.sessionDebug("DBRef "+ refNumber+ " "+source);
	    												if (source  != null && "PDB".equalsIgnoreCase(source))
	    													{
	    														String accID = ref.getAccessionId();
	    														if( accID == null || accID.length()<1) continue;
	    														accID = accID.trim().toLowerCase().intern();
	    														this.logger.sessionDebug("ACCID  "+accID);
	    														if( accID != null)
	    															{
	    																for ( Enumeration<String> keys = sites.keys(); keys.hasMoreElements();)
	    																	{
	    																		String sitePDBID = keys.nextElement();
	    																		if( sitePDBID == null) continue;
	    																
	    																		int underPos = sitePDBID.indexOf("_");
	    	    														
	    																		this.logger.sessionDebug("position underscore "+underPos +" "+sitePDBID.substring(underPos+1, name.length()));
	    																		if( underPos>0  && (sitePDBID.substring(underPos+1, sitePDBID.length()).length()==1 || sitePDBID.substring(underPos+1, sitePDBID.length()).length()==0))
	    																			{//remove chain name and leave the PDB ID
	    																				sitePDBID = sitePDBID.substring(0, underPos);
	    																			}
	    																		sitePDBID = sitePDBID.trim().toLowerCase().intern();
	    																		if( sitePDBID.equalsIgnoreCase(accID))
	    																			{
	    																				root = this.addActiveSite(root, sites.get(accID));
	    																			}
	    																	}
	    																//sites.remove(accID);
	    															}
	    													}
	    											}
    											}
	    							}
    							}
    							if( hasAnnotationToUpdate)
    								{
			    						this.logger.sessionDebug("Testing "+name+ " for residue annotation.");
			    						if(residueAnnotations.containsKey(name))
			    							{//found the alignment associated to an Active Site
			    								//add the active site as annotation element
			    								this.logger.sessionDebug("The sequence "+name+" has a residue annotation.");
			    								//AlignmentAnnotations[] annotations = root.getAlignmentAnnotations();
			    								//if (annotations == null)
			    								//	{
			    								this.addResidueAnnotation(root, residueAnnotations.remove(name));
			    								//	}
			    							}
			    						else
			    							{
			    								//look for PDB ID
			    								//usely active sites are associated with PDB chain.
			    								//check any seq has a PDB ID with an active site.
			    								Sequence seqRef = (Sequence)seq.getRefid();
			    								if (!this.logger.isProduction()) this.logger.sessionDebug(" residueAnnotations "+residueAnnotations);
			    							
			    								if (seqRef != null)
			    									{
			    										boolean found = false;
			    										int refNumber = seqRef.getDbRefCount() ;
			    										while (!found && refNumber>0)
			    											{
			    												refNumber --;
			    												DbRef ref = seqRef.getDbRef(refNumber);
			    												if( ref == null ) continue;
			    												String source = ref.getSource();
			    												this.logger.sessionDebug("DBRef "+ refNumber+ " "+source);
			    												if (source  != null && "PDB".equalsIgnoreCase(source))
			    													{
			    														String accID = ref.getAccessionId();
			    														if( accID == null || accID.length()<1) continue;
			    														accID = accID.trim().toLowerCase().intern();
			    														this.logger.sessionDebug("ACCID  "+accID);
			    														if( accID != null && residueAnnotations.containsKey(accID))
			    															{
			    																root = this.addResidueAnnotation(root, residueAnnotations.get(accID));
			    															}
			    													}
			    											}
			    										
			    									}
			    							}
			    						
			    					}
    					}
    					
    			}
    		else
    			{
    				this.logger.sessionInfo("Update Alignment: no Residue alignment provided.");
    			}
        		}catch (Exception e)
        			{
        				this.logger.sessionError("Error during Alignment update ",e);
        			}
    	
    	}
    
    /**
     * add an active site to the document
     * @param sites
     */
    private Alignment addActiveSite(Alignment align, ArrayList<AVActiveSite> sites )
    	{
    		this.logger.sessionDebug("adding active site");
    		if(align == null )
    			{
    				this.logger.sessionWarn("AddActiveSite: no Aligmnemt XML Entity.");
    				return align;
    			}
    		if( sites == null || sites.size()<1)
    			{
    				this.logger.sessionDebug("addActiveSite: No site to add.");
    				return align;
    			}
    		Hashtable <String, ArrayList<AVActiveSite>> orderedSites = new Hashtable <String,  ArrayList<AVActiveSite>>();
    		//order site per position
    		this.logger.sessionDebug(" Ordering sites: "+sites.size());
    		for (int siteNumber = sites.size() -1; siteNumber>=0; siteNumber--)
    			{
    				AVActiveSite site = sites.get(siteNumber);
    				if (site == null) continue;
    				
    				//we suppose we can have several sites at the same position
    				//orderedSites.put(Integer.valueOf(siteNumber), site);
    				Integer position = Integer.valueOf(site.getPosition());
    				this.logger.sessionDebug("position: "+position.toString());
    				 ArrayList<AVActiveSite> s = orderedSites.get(position.toString().intern());
    				 if(s == null )
    					 {
    						 s = new ArrayList<AVActiveSite>();
    					 }
    				if (!s.contains(site)) s.add(site);
    				orderedSites.put(position.toString().intern(), s);
    				//orderedSites.
    				//site.getPosition();
    			}//check if active site are already contained in the Annotation
    		for (int alignNumber = align.getAlignmentAnnotationsCount() - 1 ; alignNumber>=0; alignNumber--)
    			{
    				AlignmentAnnotations annotation= align.getAlignmentAnnotations(alignNumber);
    				if( annotation == null) continue;
    				this.logger.sessionDebug("annotation number : "+alignNumber);
    				this.logger.sessionDebug("Element count "+annotation.getAnnotationElementCount());
    				for ( int elementNumber = annotation.getAnnotationElementCount() - 1;elementNumber>=0; elementNumber-- )
    					{
    						AnnotationElement element = annotation.getAnnotationElement(elementNumber);
    						this.logger.sessionDebug("element "+elementNumber);
    	    				if( element == null) continue;
    	    			//	if(this.activeSiteDescription.equalsIgnoreCase(element.getDescription())&&
    	    						//
    	    				Integer pos = Integer.valueOf(element.getPosition());
    	    				String desc = element.getDescription();
    	    				this.logger.sessionDebug("element position "+ pos+",");
    	    				this.logger.sessionDebug("positions "+orderedSites.keys());
    	    				if(desc != null && desc.startsWith(this.activeSiteDescription) && orderedSites.containsKey(pos.toString().intern()))
    	    					{
    	    						this.logger.sessionInfo("AddActiveSite: The active site already exists at position: "+element.getPosition());
    	    						ArrayList<AVActiveSite> s = orderedSites.remove(pos.toString().intern());
    	    						
    	    						//continue;
    	    					}
    	    				pos = null;
    					}
    			}
    		//the remaining sites must be added.
    		if(orderedSites !=null && orderedSites.size()>0 )
    			{
    				AlignmentAnnotations annotation = new AlignmentAnnotations();
    				
    				Collection<ArrayList<AVActiveSite>> s =  orderedSites.values();
    				if( s != null && s.size()>0)
    					{
    						for ( Iterator <ArrayList<AVActiveSite>> iter = s.iterator(); iter.hasNext();)
    							{
    								ArrayList<AVActiveSite> list = iter.next();
    								if (list == null || list.size()<1) continue;
    								for (int arrayNumber = list.size()-1; arrayNumber>=0; arrayNumber--)
    									{
    										AVActiveSite site = list.get(arrayNumber);
    										AnnotationElement element = new AnnotationElement();
    										if (site == null ) continue;
    										this.logger.sessionDebug("site : "+site.getSequenceName() + " "+site.getPosition() );
    			    						element.setDescription(this.activeSiteDescription +" "+site.getLigandName());
    			    						element.setPosition(site.getPosition());
    			    						//element.setValue();
    			    						annotation.addAnnotationElement(element);
    									}
    							}
    					}
    				/*for (Enumeration siteEnum = ; siteEnum.hasMoreElements();)
    					{
    						AVActiveSite site = (AVActiveSite)siteEnum.nextElement();
    						if (site == null) continue;
    						logger.Applidebug("site : "+site.getSequenceName());
    						element.setDescription(this.activeSiteDescription +" "+site.getLigandName());
    						element.setPosition(site.getPosition());
    						//element.setValue();
    						annotation.addAnnotationElement(element);
    					}*/
    				//adding provenance entry to the annotation
    				Provenance provenance = annotation.getProvenance();
    				Entry provenanceEntry = this.createProvenanceEntry(this.client, "Update - added ActiveSite");
    				if(provenanceEntry != null )//otherwise  something is wrong
    					{
    						if( provenance == null) provenance = new Provenance();
    						provenance.addEntry(provenanceEntry);
    					}
    				align.addAlignmentAnnotations(annotation);
    			}
    		return align;
    	}

    /**
     * add an active site to the document
     * @param sites
     */
    private Alignment addResidueAnnotation(Alignment align, ArrayList<ResidueAnnotation> annotations )
    	{
    		this.logger.sessionDebug("adding active site");
    		if(align == null )
    			{
    				this.logger.sessionWarn("AddResidueAnnotation: no Aligmnemt XML Entity.");
    				return align;
    			}
    		if( annotations == null || annotations.size()<1)
    			{
    				this.logger.sessionDebug("addResidueAnnotation: No annotation to add.");
    				return align;
    			}
    		Hashtable <String, ArrayList<ResidueAnnotation>> orderedAnnotation = new Hashtable <String,  ArrayList<ResidueAnnotation>>();
    		//order site per position
    		this.logger.sessionDebug(" Ordering sites: "+annotations.size());
    		for (int siteNumber = annotations.size() -1; siteNumber>=0; siteNumber--)
    			{
    				ResidueAnnotation annotation = annotations.get(siteNumber);
    				if (annotation == null) continue;
    				
    				//we suppose we can have several sites at the same position
    				//orderedSites.put(Integer.valueOf(siteNumber), site);
    				Integer position = Integer.valueOf(annotation.getResidueNumber());
    				this.logger.sessionDebug("position: "+position.toString());
    				 ArrayList<ResidueAnnotation> s = orderedAnnotation.get(position.toString().intern());
    				 if(s == null )
    					 {
    						 s = new ArrayList<ResidueAnnotation>();
    					 }
    				if (!s.contains(annotation)) s.add(annotation);
    				orderedAnnotation.put(position.toString().intern(), s);
    				//orderedSites.
    				//site.getPosition();
    			}//check if active site are already contained in the Annotation
    		for (int alignNumber = align.getAlignmentAnnotationsCount() - 1 ; alignNumber>=0; alignNumber--)
    			{
    				AlignmentAnnotations annotation= align.getAlignmentAnnotations(alignNumber);
    				if( annotation == null) continue;
    				this.logger.sessionDebug("annotation number : "+alignNumber);
    				this.logger.sessionDebug("Element count "+annotation.getAnnotationElementCount());
    				for ( int elementNumber = annotation.getAnnotationElementCount() - 1;elementNumber>=0; elementNumber-- )
    					{
    						AnnotationElement element = annotation.getAnnotationElement(elementNumber);
    						this.logger.sessionDebug("element "+elementNumber);
    	    				if( element == null) continue;
    	    			//	if(this.activeSiteDescription.equalsIgnoreCase(element.getDescription())&&
    	    						//
    	    				Integer pos = Integer.valueOf(element.getPosition());
    	    				String desc = element.getDescription();
    	    				this.logger.sessionDebug("element position "+ pos+",");
    	    				if( !this.logger.isProduction()) this.logger.sessionDebug("positions "+orderedAnnotation.keys());
    	    				if(desc != null && desc.startsWith(this.activeSiteDescription) && orderedAnnotation.containsKey(pos.toString().intern()))
    	    					{
    	    						this.logger.sessionInfo("AddActiveSite: The active site already exists at position: "+element.getPosition());
    	    						ArrayList<ResidueAnnotation> s = orderedAnnotation.remove(pos.toString().intern());
    	    						
    	    						//continue;
    	    					}
    	    				pos = null;
    					}
    			}
    		//the remaining sites must be added.
    		if(orderedAnnotation !=null && orderedAnnotation.size()>0 )
    			{
    				AlignmentAnnotations annots = new AlignmentAnnotations();
    				
    				Collection<ArrayList<ResidueAnnotation>> s =  orderedAnnotation.values();
    				if( s != null && s.size()>0)
    					{
    						for ( Iterator <ArrayList<ResidueAnnotation>> iter = s.iterator(); iter.hasNext();)
    							{
    								ArrayList<ResidueAnnotation> list = iter.next();
    								if (list == null || list.size()<1) continue;
    								for (int arrayNumber = list.size()-1; arrayNumber>=0; arrayNumber--)
    									{
    										ResidueAnnotation annotation = list.get(arrayNumber);
    										AnnotationElement element = new AnnotationElement();
    										if (annotation == null ) continue;
    										this.logger.sessionDebug("annotation : "+annotation.getSequenceName() + " "+annotation.getResidueNumber() );
    			    						element.setDescription(annotation.getDescription());
    			    						element.setPosition(annotation.getResidueNumber());
    			    						if (annotation.getDisplayCharacter()!= null && annotation.getDisplayCharacter().length()>0)
    			    							element.setDisplayCharacter(annotation.getDisplayCharacter());
    			    						else
    			    							element.setDisplayCharacter("?");
    			    						if( annotation.getSecondaryStructure() != null && annotation.getSecondaryStructure().length()>0 )
    			    							{
    			    								element.setSecondaryStructure(annotation.getSecondaryStructure());
    			    							}
    			    						else
    			    							element.setSecondaryStructure("?");
    			    						element.setValue(Float.parseFloat(annotation.getValue()));
    			    						//element.setValue();
    			    						annots.addAnnotationElement(element);
    									}
    							}
    					}
    				/*for (Enumeration siteEnum = ; siteEnum.hasMoreElements();)
    					{
    						AVActiveSite site = (AVActiveSite)siteEnum.nextElement();
    						if (site == null) continue;
    						logger.Applidebug("site : "+site.getSequenceName());
    						element.setDescription(this.activeSiteDescription +" "+site.getLigandName());
    						element.setPosition(site.getPosition());
    						//element.setValue();
    						annotation.addAnnotationElement(element);
    					}*/
    				//adding provenance entry to the annotation
    				Provenance provenance = annots.getProvenance();
    				Entry provenanceEntry = this.createProvenanceEntry(this.client, "Update - added Residue Annotation");
    				if(provenanceEntry != null )//otherwise  something is wrong
    					{
    						if( provenance == null) provenance = new Provenance();
    						provenance.addEntry(provenanceEntry);
    					}
    				align.addAlignmentAnnotations(annots);
    			}
    		return align;
    	}
    
    private Hashtable<String, Object>  addParameters(Hashtable<String, Object> newParameters, Hashtable<String, Object> parameters)
    	{
    		if (parameters == null) parameters = new Hashtable<String, Object>();
    		if (newParameters != null && newParameters.size()>0)
    			{
    				boolean found = false;
    				/*for (Enumeration keys = newParameters.keys(); keys.hasMoreElements();)
    					{
    						String key = (String) keys.nextElement();
    						if( key == null) continue;
    						Object value = newParameters.get(key);
    						if (value == null) continue;
    						
    						String newKey = null;
    						if("molecule".equals(key) )
    							{
    								if( parameters.containsValue(value)) continue;
    								newKey = "molecule"+this.moleculeNumber;
    							}
    						else
    							if("name".equals(key) )
    								{
    									if( parameters.containsValue(value)) continue;
    									newKey = "name"+this.moleculeNumber;
    								}
    							else newKey = key;
    						parameters.put(newKey, value);
    						logger.Applidebug("Adding new parameter "+newKey);
    						newParameters.remove(key);
    						key = null;
    						found = true;
    					}
    				found = false;
    				this.moleculeNumber++;*/
    				boolean moreMolecules = true;
    				int molNumber = -1;
    				int highlightN= -1;
    				boolean moreHighlight = true;
    				if (!this.logger.isProduction() )
    					{
    						for (Enumeration<String> keys = newParameters.keys(); keys.hasMoreElements();)
    							{
    								this.logger.sessionDebug("new parameter "+keys.nextElement());
    							}
    						for (Enumeration<String> keys = parameters.keys(); keys.hasMoreElements();)
    							{
    								this.logger.sessionDebug("current stored parameter "+keys.nextElement());
    							}
    					}
    				while (newParameters != null && !newParameters.isEmpty() && (moreMolecules || moreHighlight))
    					{
    						highlightN++;
    						String currentParameterName = "highlightT"+highlightN;
    						Object hig  = newParameters.remove(currentParameterName);
    						currentParameterName = null;
    						if(hig != null)
    							{
    								if (!parameters.containsValue(hig))
    									{
    										this.logger.sessionDebug("Highlight found in the document");
    								//newParameters.remove("highlight"+highlightNumber);
    										int currentNumber = this.highlightNumber++;
    										String newKey = "highlight"+currentNumber;
    										this.logger.sessionDebug("Adding new parameter "+newKey);
    										parameters.put(newKey, hig);
    									}
    								else
    									{
    										this.logger.sessionDebug("the highlight is already contained in the parameter list.");
    									}
    							}
    						else
    							{
    								moreHighlight = false; 
    								//continue;
    								this.logger.sessionDebug("no more highlight");
    							}
    						
    						molNumber++;
    						Object mol  = newParameters.get("moleculeG"+molNumber);
    						if (mol == null) 
    							{
    								moreMolecules = false; 
    								continue;
    							}
    						newParameters.remove("moleculeG"+molNumber);
    						Object name  = newParameters.get("nameG"+molNumber);
    						if (name == null) 
    							{
    								//moreMolecules = false; 
    								continue;
    							}
    						newParameters.remove("nameG"+molNumber);
    						if( parameters.containsValue(mol)) continue;
    						int currentNumber = this.moleculeNumber++;
    						String newKey = "molecule"+currentNumber;
    						this.logger.sessionDebug("Adding new parameter "+newKey);
    						parameters.put(newKey, mol);
    						if( parameters.containsValue(name)) continue;
    						newKey = "name"+currentNumber;
    						this.logger.sessionDebug("Adding new parameter "+newKey);
    						parameters.put(newKey, name);
    					}
    				if (newParameters!= null && !newParameters.isEmpty())parameters.putAll(newParameters);
    			}
    		if (newParameters != null)
    			{
    				newParameters.clear();
    				newParameters = null;
    			}
    		return parameters;
    	}
 
    /** @poseidon-generated */
    public SequenceGrouping getSequenceGrouping() {
        return this.groupingManager;
    }
    /** @poseidon-generated */
    public void setSequenceGrouping(SequenceGrouping sequenceGrouping) {
        this.groupingManager = sequenceGrouping;
    }
    
    
    private Entry createProvenanceEntry(IClient client, String action)
    	{
    		if (client == null)
    			{
    				this.logger.sessionError("No vamsas client provided to create provenance entry. The entry object will not be added.");
    				return null;
    			}
    		return this.createProvenanceEntry(client.getUserHandle().getFullName(), action);
    	}
    
    private Entry createProvenanceEntry(String user, String action)
    	{
    		if( user == null || user.length()<1)
    			{
    				this.logger.sessionError("No User name provided to create provenance entry. The entry object will not be added.");
    				return null;
    			}
    		if( action == null || action.length()<1)
    			{
    				this.logger.sessionError("No action provided to create provenance entry. The entry object will not be added.");
    				return null;
    			}
    		Entry provenanceEntry = new Entry();
    		provenanceEntry.setUser(user);
    		provenanceEntry.setAction(action);
    	
    		/* 
    		 ** on some JDK, the default TimeZone is wrong
    		 ** we must set the TimeZone manually!!!
    		 **   Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("EST"));
    		 */
    		Calendar cal = Calendar.getInstance(TimeZone.getDefault());
    		org.exolab.castor.types.Date date = new org.exolab.castor.types.Date (cal.getTime());
    		provenanceEntry.setDate(date);
    		cal = null;
    		return provenanceEntry;
    	}
    
    public ArrayList<String> getGroupingModes()
    	{
    		ArrayList<String> modes = new ArrayList<String>();
    		modes.add("structure");
    		modes.add("Protein domain");
    		return modes;
    	}
    
    /**
     * Create a document following provided parameters
     * 
     * @param parameters list of parameters, containing objects to map to the document objects/tree
     * @return the IClient with the associated newly created document
     * @throws IOException if an  error occurs during VAMsas document file access
     */
    public IClient createDocument(Hashtable<String, Object> parameters) throws IOException
    	{
    		if(parameters == null  || parameters.size()<1)
    			{
    				this.logger.sessionError("No parameter provided to create the VAMSAS document. Abort.");
    			}
    		 IClientDocument _doc = null;
    		
			 VamsasDocument doc = new VamsasDocument();
    		 VAMSAS root  = new VAMSAS();
    		 doc.addVAMSAS(root);
    		 
    		 
    		 Tree aTree = this.createTree(parameters);
    		 if( aTree != null) root.addTree(aTree);
    		 
    		DataSet set = this.createDataSet(parameters);
    		if (set != null)
    			root.addDataSet(set);
    		 
			IdFactory vorba = null;
			   // TODO: reduce size of vorba ids generated from these parameters to IdFactory (mainly sessionHandle rationalization ?)
			   try {
					vorba= new IdFactory(this.client.getSessionHandle(),
					 this.client.getClientHandle(), ((VAMAVClient)this.client).getUserHandle());//makeVorbaIdFactory();
				 // if session currently holds data - read it in - or get a dummy
					this.logger.sessionDebug("Accessing document");
				  }
			   catch (Exception e) {
			   	File sessionDir = ((VAMAVClient)this.client).getVAMAVSession().getSessionDirectory();
					this.logger.sessionError("Failed to get session document for session directory '"+sessionDir.getAbsolutePath()+"'", e);
				 throw new IOException("Failed to get session document for session directory '"+sessionDir.getAbsolutePath()+"'");
			   }
			   // Construct the IClientDocument instance
			ApplicationData appliData = new ApplicationData();
			User user = new User();
			user.setFullname(this.client.getUserHandle().getFullName());
			user.setOrganization(this.client.getUserHandle().getOrganization());
		
			appliData.addUser(user);
    		doc.addApplicationData(appliData);
			_doc = new ClientDocument(doc, (VamsasArchive)null, vorba, this.client);
    		 
    		//_doc = new ClientDocument(doc, new VAMSAS[]{root}, null,this.client);
    		this.client.updateDocument(_doc);
    		return this.client;
    	}
    
    /** Create Vamsas Tree object
    * Currently do nothing
    * @return the created tree 
    * @param parameters a list of parameters from an AstexViewer@MSD-EBI client
    */
   private Tree createTree(Hashtable<String, Object> parameters)
	   {
		   return null;
   		}
   
   
   private DataSet createDataSet(Hashtable<String, Object> parameters) 
   	{
   		if (parameters == null || parameters.size()<1)
   			{
					this.logger.sessionDebug("No parameter provided to create DataSet");
					return null;
   			}
   		this.logger.sessionDebug("Create DataSet ");
   		DataSet set = new DataSet();
   		Alignment align = this.createAlignment(parameters);
   		if (align != null) set.addAlignment(align);
 
   		set  = this.createSequences(parameters,set);
   
   		this.logger.sessionDebug("end CreateDataSet ");
   		return set;
   	}
   	
   /**
    * creates a Alignment entity following the given parameters
    * @param parameters  list of objects used to create  the Alignment  tree
    */
   private Alignment createAlignment(Hashtable<String, Object> parameters) 
   	{        
   		this.logger.sessionDebug("Create Alignment");
   		if( parameters == null || parameters.size()<1)
   			{
   				this.logger.sessionWarn("createAlignment: no object provided: Can not create.");
   				return  null;
   			}
   		Alignment align = null;
   		//new Alignment();
   		if (!this.logger.isProduction())  this.logger.sessionDebug("list of parameters: "+parameters);	
   		return align;
   	}
   
   /**
    * creates a Alignment entity following the given parameters
    * @param parameters  list of objects used to create  the Alignment  tree
    */
   private DataSet createSequences(Hashtable<String, Object> parameters, DataSet set) 
   	{        
   		this.logger.sessionDebug("Create Sequences");
   		if( parameters == null || parameters.size()<1)
   			{
   				this.logger.sessionWarn("createSequence: no object provided: Can not create.");
   				return  null;
   			}
   		ArrayList<AVSequence> sequences = (ArrayList<AVSequence>) parameters.get("MSD.vamav.common.AVSequence");
   		if( sequences != null && sequences.size()>0)
   			{
   				Alignment align = new Alignment();
   				
		   		for (int sequenceNumber = sequences.size() -1; sequenceNumber >=0; sequenceNumber-- )
		   			{
		   				AVSequence sequence = sequences.get(sequenceNumber);
		   				
		   				//create sequence object
		   				Sequence seq = new Sequence();
		   				seq.setName(sequence.getName());
		   				String sequenceWithoutGap = sequence.getSequence();
		   				if( sequenceWithoutGap != null && sequenceWithoutGap.length()>0)
		   					{
		   						sequenceWithoutGap = sequenceWithoutGap.replaceAll("-", "");
		   					}
		   				seq.setSequence(sequenceWithoutGap);
		   				set.addSequence(seq);
		   				
		   				//create Alignment object relative to thie Sequence object
		   			
		   				AlignmentSequence alignmSeq = new AlignmentSequence();
		   				alignmSeq.setName(sequence.getName());
		   				alignmSeq.setSequence(sequence.getSequence());
		   				alignmSeq.setRefid(seq);
		   				align.addAlignmentSequence(alignmSeq);
		   			}
		   		set.addAlignment(align);
   			}
   		
   		if (!this.logger.isProduction())  this.logger.sessionDebug("list of parameters: "+parameters);	
   		return set;
   	}
  
  
 }
